package org.gcube.phd;

import static org.gcube.common.authorization.client.Constants.authorizationService;

import java.util.HashMap;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import org.gcube.common.authorization.client.Constants;
import org.gcube.common.authorization.library.AuthorizationEntry;
import org.gcube.common.authorization.library.provider.SecurityTokenProvider;
import org.gcube.common.authorization.library.provider.UserInfo;
import org.gcube.common.scope.api.ScopeProvider;
import org.gcube.common.scope.impl.ScopeBean;
import org.gcube.common.scope.impl.ScopeBean.Type;
import org.gcube.context.ContextElaborator;

public class StatisticsCollector extends ContextElaborator {
	
	private Map<Type,Integer> contextCount;
	private SortedMap<ScopeBean, ContextStatistic> sortedMap; 
	
	public StatisticsCollector() {}
	
	@SuppressWarnings("unused")
	private String generateUserToken(String context) throws Exception {
		AuthorizationEntry authorizationEntry = Constants.authorizationService()
				.get(SecurityTokenProvider.instance.get());
		UserInfo userInfo = (UserInfo) authorizationEntry.getClientInfo();
		String userToken = authorizationService().generateUserToken(userInfo, context);
		logger.trace("Token for Context {} for {} is {}", context, userInfo.getId(), userToken);
		return userToken;
	}
	
	
	@Override
	public void elaborateContext(ScopeBean scopeBean) throws Exception {
		Type type = scopeBean.type();
		Integer integer = contextCount.get(type);
		if(integer == null) {
			integer = 0;
		}
		contextCount.put(type, integer + 1);
		
		
		String contextFullName = scopeBean.toString();
		/*
		String token = generateUserToken(contextFullName);
		ContextTest.setContext(token);
		*/
		ScopeProvider.instance.set(contextFullName);
		
		ContextStatistic contextStatistic = new ContextStatistic(scopeBean);
		sortedMap.put(scopeBean, contextStatistic);
	}
	
	public SortedMap<ScopeBean, ContextStatistic> getStatistics() throws Exception{
		if(sortedMap == null) {
			contextCount = new HashMap<>();
			sortedMap = new TreeMap<>();
			all();
		}
		return sortedMap;
	}
	
	public Map<Type,Integer> getContextCount() throws Exception {
		if(contextCount == null) {
			contextCount = new HashMap<>();
			sortedMap = new TreeMap<>(new ScopeBeanComparator());
			all();
		}
		return contextCount;
	}
	
}
